-- CombatCam by Everglow

-- View 1 = 1st person, accessed by slash command ("/ccam zin")
-- View 2 = Normal
-- View 3 = Mounted View
-- View 4 = Combat
-- View 5 = User defined, accessed by slash command ("/ccam  zout")

-- local vars
local CC_Ver="3.0.02"
local CCTag="|c00FFFF66CombatCam "..CC_Ver.."|r: "
local infotext =
	"Instructions:\n"..
	"adjust your camera as desired then click a button to save that view.\n\n"..
	"Slash commands:\n"..
	"/combatcam: display this window\n"..
	"/combatcam zin: first person view\n"..
	"/combatcam zout: custom view"

local useCC, useMC, useSE
local wim_up = false
local wim_box
local firstRun = false

-- local functions
local function ChatMsg(msg)
	DEFAULT_CHAT_FRAME:AddMessage(msg)
end

local function BtoStr(bool)
	if(bool) then return "true" else return "false" end
end

local function setCam(view)
	if view>0 and view<6 then
		SetView(view)
	elseif view == 0 then
		if IsMounted() or IsFlying() then
			SetView(3)
		elseif UnitAffectingCombat("player") then
			SetView(4)
		else
			SetView(2)
		end
	else
		ChatMsg(CCTag..format("|c00FF0000Error: |c00CC6666invalid view (%d)- please report this error where you downloaded CombatCam. "..
							" Please include in your report what you were doing when the error occured.",view))
	end
end

local function acceptChanges()
	if firstRun then
		CombatCamInfoText:SetText(infotext)
		firstRun = false
	end
	useCombatCam = useCC;
	useMountCam = useMC;
	useSmartEdit = useSE;
end

local function cancelChanges()
	if firstRun then
		CombatCamInfoText:SetText(infotext)
		firstRun = false
	end
	useCC = useCombatCam;
	useMC = useMountCam;
	useSE = useSmartEdit;
	CombatCamEnableCheckButton:SetChecked(useCC);
	MountCamEnableCheckButton:SetChecked(useMC);
	SmartEditEnableCheckButton:SetChecked(useSE);
end

-- global functions
function CombatCamToggle()
	useCC = not useCC;
end

function MountCamToggle()
	useMC = not useMC;
end

function SmartEditToggle()
	useSE = not useSE;
end

-- SLASH COMMANDS
function CombatCam_SlashCommandHandler(a)
	if (a == "zin") then
		SetView(1)
	elseif (a == "zout") then
		SetView(5)
	else
		InterfaceOptionsFrame_OpenToCategory(CombatCamFrame)
	end
end

function CombatCam_OnLoad(frame)
	frame.name = "CombatCam"
	frame.okay = acceptChanges
	frame.cancel = cancelChanges
	frame:RegisterEvent("PLAYER_LOGIN")
	frame:RegisterEvent("COMPANION_UPDATE")
	frame:RegisterEvent("UNIT_SPELLCAST_SUCCEEDED")
	frame:RegisterEvent("PLAYER_REGEN_DISABLED")
	frame:RegisterEvent("PLAYER_REGEN_ENABLED")
	InterfaceOptions_AddCategory(frame)

	SLASH_COMBATCAM1 = "/combatcam"
	SLASH_COMBATCAM2 = "/ccam"
	SlashCmdList["COMBATCAM"] =
	function(msg)
		CombatCam_SlashCommandHandler(msg)
	end
	getglobal(CombatCamEnableCheckButton:GetName() .. "Text"):SetText("Enable CombatCam");
	getglobal(MountCamEnableCheckButton:GetName() .. "Text"):SetText("Enable MountCam");
	getglobal(SmartEditEnableCheckButton:GetName() .. "Text"):SetText("Enable SmartEdit");
	CombatCamInfoText:SetWidth("300")
end
	
function CombatCam_OnEvent(frame, event, ...)
	if (event == "PLAYER_LOGIN") then
		if useCombatCam == nil then  -- initialize vars
			firstRun = true
			local infotext1 = "|c0000FFFFIt looks like this is your first time running CombatCam on this toon. "..
				"Please set your views if you have not already done so on another toon.|r\n\n\n"
			useCombatCam = false
			useMountCam = false
			useSmartEdit = false
			CombatCamInfoText:SetText(infotext1..infotext)
			InterfaceOptionsFrame_OpenToFrame(frame)
		else
			CombatCamEnableCheckButton:SetChecked(useCombatCam)
			MountCamEnableCheckButton:SetChecked(useMountCam)
			SmartEditEnableCheckButton:SetChecked(useSmartEdit)
			CombatCamInfoText:SetText(infotext)
		end
		-- initialize local vars
		useCC = useCombatCam
		useSE = useSmartEdit
		if useCC or (TitanRiderAutoCam_Frame and TR_UseAutoCam) then setCam(0) end
		ChatMsg(CCTag.."Loaded")
	elseif event == "PLAYER_REGEN_DISABLED" then	--player enters combat
		if useCombatCam then 
			if not (IsMounted() or IsFlying()) then
				setCam(0)
			end
		end
		if useSmartEdit then
			if WIM_EditBoxInFocus then
				wim_box = WIM_EditBoxInFocus
				wim_up = true
				wim_box:ClearFocus()
			else
				ChatFrameEditBox:ClearFocus()
				ChatFrameEditBox:SetAutoFocus(false)
			end
		end
	elseif event == "PLAYER_REGEN_ENABLED" then	--player leaves combat
		if useCombatCam then setCam(0) end
		if useSmartEdit then 
			if (wim_up) then
				wim_box:SetFocus()
				wim_up = false
			else
				ChatFrameEditBox:SetAutoFocus(true)
			end
		end
	elseif event == "COMPANION_UPDATE" then -- possibly became mounted
		companionType = ...;
		if useMountCam then
			if companionType == "MOUNT" then setCam(0) end
		end
	elseif event == "UNIT_SPELLCAST_SUCCEEDED" then -- possibly  switched to a traveling form
		whoCast,spName = ...;
		if useMountCam then
			if whoCast == "player" and (spName == "Flight Form" or spName == "Swift Flight Form" or spName == "Travel Form" or spName == "Ghost Wolf") then
				local i, buff = 1, UnitBuff("player", 1)
				while buff do
					if buff == spName then break end
					i = i + 1;
					buff = UnitBuff("player", i);
				end;
				if not buff then setCam(0) else setCam(3) end
			end
		end
	end
end
